package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.params.GOST3410KeyParameters;
import cn.com.infosec.crypto.params.GOST3410Parameters;
import cn.com.infosec.crypto.params.GOST3410PrivateKeyParameters;
import cn.com.infosec.crypto.params.GOST3410PublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.math.BigInteger;
import java.security.SecureRandom;

public class GOST3410Signer implements DSA {
   GOST3410KeyParameters key;
   SecureRandom random;

   public void init(boolean forSigning, CipherParameters param) {
      if (forSigning) {
         if (param instanceof ParametersWithRandom) {
            GOST3410Signer var10000 = this;
            ParametersWithRandom var3;
            this.random = (var3 = (ParametersWithRandom)param).getRandom();
            var10000.key = (GOST3410PrivateKeyParameters)var3.getParameters();
         } else {
            this.random = new SecureRandom();
            this.key = (GOST3410PrivateKeyParameters)param;
         }
      } else {
         this.key = (GOST3410PublicKeyParameters)param;
      }

   }

   public BigInteger[] generateSignature(byte[] message) {
      int var2;
      byte[] var3 = new byte[var2 = message.length];

      for(int var4 = 0; var4 != var2; ++var4) {
         var3[var4] = message[var2 - 1 - var4];
      }

      BigInteger var5;
      var5 = new BigInteger.<init>(1, var3);
      GOST3410Parameters var6 = this.key.getParameters();

      BigInteger var10000;
      BigInteger message1;
      do {
         var10000 = message1 = new BigInteger;
         var2 = var6.getQ().bitLength();
         message1.<init>(var2, this.random);
      } while(var10000.compareTo(var6.getQ()) >= 0);

      var10000 = message1;
      message1 = var6.getA().modPow(message1, var6.getP()).mod(var6.getQ());
      BigInteger var7 = var10000.multiply(var5).add(((GOST3410PrivateKeyParameters)this.key).getX().multiply(message1)).mod(var6.getQ());
      return new BigInteger[]{message1, var7};
   }

   public boolean verifySignature(byte[] message, BigInteger r, BigInteger s) {
      int var4;
      byte[] var5 = new byte[var4 = message.length];

      for(int var6 = 0; var6 != var4; ++var6) {
         var5[var6] = message[var4 - 1 - var6];
      }

      BigInteger message1;
      message1 = new BigInteger.<init>(1, var5);
      GOST3410Parameters var8 = this.key.getParameters();
      BigInteger var9;
      if ((var9 = BigInteger.valueOf(0L)).compareTo(r) < 0 && var8.getQ().compareTo(r) > 0) {
         if (var9.compareTo(s) < 0 && var8.getQ().compareTo(s) > 0) {
            BigInteger var10003 = message1;
            message1 = var8.getQ().subtract(new BigInteger("2"));
            s = s.multiply(message1 = var10003.modPow(message1, var8.getQ())).mod(var8.getQ());
            message1 = var8.getQ().subtract(r).multiply(message1).mod(var8.getQ());
            return var8.getA().modPow(s, var8.getP()).multiply(((GOST3410PublicKeyParameters)this.key).getY().modPow(message1, var8.getP())).mod(var8.getP()).mod(var8.getQ()).equals(r);
         } else {
            return false;
         }
      } else {
         return false;
      }
   }
}
